////////////////////////////////////////////////////////////////////
// <VXPLib SDK>
////////////////////////////////////////////////////////////////////
// This optional class library can be used in Microsoft Visual C++
// version 6 and 7 to facilitate access to the VXPLib component.
//
// The class library contained in this file is intellectual property
// of Vitaly Tomilov.
//
// Last modified: 18/10/2002
////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////
// <Class Library Specification>
////////////////////////////////////////////////////////////////////
// The library contains 9 classes - each represents a COM interface
// from the VXPLib component: CVXPFont, CVXPMargins, CVXPSound,
// CVXPSettings, CVXPQuickHTML, CVXPTooltip, CVXPCmdToolList,
// CVXPWndToolList and CVXPTooltipManager.
//
// It also contains a few service classes and structures used within
// the class library whose names are prefixed with double underscore,
// but their meaning and functionality are just to hide overhead
// of complexity of using COM in C++, and should be ignored.
//
// Wherever it was possible the class library was designed to be 1:1
// with the COM object model implemented in VXPLib. Therefore, using
// this library will be very similar to that in a script environment
// like Visual Basic.
//
// This header file is not neccessary to include in your C++ project,
// but it is recommended to do so, if you want the COM library
// to be accessible seamlessly and in a secure way. This library
// offers a safe way of instatiating and releasing COM interfaces
// of the VXPLib component. Whenever an object goes out of scope it
// is garanteed to release the COM interface it represents.
//
// Probably the main power of this class library as opposed to using
// pure C++ COM support is the events handling.
// Handling events in a C++ environment can be difficult to the
// majority of programmers who never dealt with COM events in C++.
// Even though in VC++ 7 were introduced C++ Attributes it still would
// require a lot of work to do in order to make your C++ library work
// more or less like the component itself does.
//
// HOW TO HANDLE EVENTS OF THE LIBRARY
//
// If you just include this file it will give you access to all features
// of the COM library but the events. In order to enable this library
// to handle all the events add definition VXPLIB_WITH_EVENTS in your
// project before including this file. For instance, you can include
// the library like this:
//
// #define VXPLIB_WITH_EVENTS
// #include "VXPLib.h"
//
// Alternatively, you can add VXPLIB_WITH_EVENTS into precompiler
// settings of your project.
// When having activated the use of events of the library it gives
// the library access to using ATL (Active Template Library) in order
// to handle COM events. This obliges you to provide support for ATL
// in your project. If you are already using ATL in your project there
// is nothing extra to be done to handle events. If you are not
// using ATL, your code modifications could like this:
// 
// 1. Adding two lines in your StdAfx.h:
//
// #define VXPLIB_WITH_EVENTS
// #include "VXPLib.h"
//
// 2. Adding one line in your StdAfx.cpp:
//
// CComModule _Module; 
//
// 3. Adding as the very first line in your App::OnInitInstance():
//
// _Module.Init(NULL, GetModuleHandle(NULL));
//
/////////////////////////////////////////////////////////////////////////
// Notes:
//
// 1. It is assumed that COM support for your project has already been
// enabled, and your GUI has entered STA (Single Thread Apartment).
// In an MFC project it is done by calling AfxOleInit() in your
// App::OnInitInstance() method as the first thing, and in a non-MFC
// application by calling CoInitialize(NULL) as the first call in the
// program (GUI thread).
//
// 2. Apart from functionality of the class library itself COM methods
// and properties are not documented in this file because they are called
// and work exactly as COM methods and COM properties they represent,
// except for return values for methods and some methods parameters,
// which were changed for easier use with C++ native types.
// Read VXPLib SDK for information on methods and properties of the COM
// interfaces contained in the library.
/////////////////////////////////////////////////////////////////////////

#ifndef VXPLib_Header
#define VXPLib_Header

#ifdef VXPLIB_WITH_EVENTS
#include <atlbase.h>
extern CComModule _Module;
#include <atlcom.h>
#endif

#import "VXPLib.dll" no_namespace named_guids

#pragma warning(push)
#pragma warning(disable:4312 4311)

#define VXP_DECLARE(T, A) __declspec(property(get=__Get##A, put=__Put##A)) T A;
#define VXP_DECLARE_READ(T, A) __declspec(property(get=__Get##A)) T A;

#define VXP_IMPLEMENT_SIMPLE(T, A, D)\
T __Get##A()\
{\
	if(m_pInterface)\
	{\
		T value;\
		m_pInterface->get_##A(&value);\
		return value;\
	}\
	return D;\
}\
void __Put##A(T NewValue)\
{\
	if(m_pInterface)\
		m_pInterface->put_##A(NewValue);\
}

#define VXP_IMPLEMENT_READ_SIMPLE(T, A, D)\
T __Get##A()\
{\
	if(m_pInterface)\
	{\
		T value;\
		m_pInterface->get_##A(&value);\
		return value;\
	}\
	return D;\
}

#define VXP_IMPLEMENT_REF(T, V)\
T & __Get##V()\
{\
	return m_##V;\
}

#define VXP_IMPLEMENT_LONG(A)	VXP_IMPLEMENT_SIMPLE(long, A, 0)
#define VXP_IMPLEMENT_SHORT(A)	VXP_IMPLEMENT_SIMPLE(short, A, 0)

#define VXP_IMPLEMENT_STR(A)\
_bstr_t __Get##A()\
{\
	if(m_pInterface)\
	{\
		BSTR value;\
		m_pInterface->get_##A(&value);\
		return _bstr_t(value);\
	}\
	return _bstr_t();\
}\
void __Put##A(_bstr_t NewValue)\
{\
	if(m_pInterface)\
		m_pInterface->put_##A(NewValue);\
}

#define VXP_IMPLEMENT_BOOL(A)\
bool __Get##A()\
{\
	if(m_pInterface)\
	{\
		VARIANT_BOOL value;\
		m_pInterface->get_##A(&value);\
		return value?true:false;\
	}\
	return false;\
}\
void __Put##A(bool NewValue)\
{\
	if(m_pInterface)\
		m_pInterface->put_##A(NewValue);\
}

#define VXP_IMPLEMENT_HANDLE(T, A)\
T __Get##A()\
{\
	if(m_pInterface)\
	{\
		long value;\
		m_pInterface->get_##A(&value);\
		return (T)value;\
	}\
	return NULL;\
}

#define VXP_IMPLEMENT_READ(T, A)\
T __Get##A()\
{\
	if(m_pInterface)\
	{\
		T value;\
		m_pInterface->get_##A(&value);\
		return value;\
	}\
	return 0;\
}

#define VXP_IMPLEMENT_READ_BOOL(A)\
bool __Get##A()\
{\
	if(m_pInterface)\
	{\
		VARIANT_BOOL value;\
		m_pInterface->get_##A(&value);\
		return value?true:false;\
	}\
	return false;\
}

////////////////////////////////////////
// This is a service class, and should
// not be used by client applications.
////////////////////////////////////////
class __vxpfont
{
friend class CVXPFont;

public:

	VXP_IMPLEMENT_BOOL(Transparent)
	VXP_IMPLEMENT_BOOL(Italic)
	VXP_IMPLEMENT_BOOL(Underlined)
	VXP_IMPLEMENT_BOOL(StrikeOut)
	VXP_IMPLEMENT_SHORT(Size)
	VXP_IMPLEMENT_SHORT(Escapement)
	VXP_IMPLEMENT_SHORT(Quality)
	VXP_IMPLEMENT_SHORT(PitchAndFamily)
	VXP_IMPLEMENT_SHORT(Orientation)
	VXP_IMPLEMENT_SHORT(OutPrecision)
	VXP_IMPLEMENT_SHORT(ClipPrecision)
	VXP_IMPLEMENT_HANDLE(HFONT, hFont)
	VXP_IMPLEMENT_SIMPLE(VXPFontWeight, Weight, fwDontCare)
	VXP_IMPLEMENT_SIMPLE(VXPCharSet, CharSet, csDEFAULT)
	VXP_IMPLEMENT_STR(Name)
	VXP_IMPLEMENT_SIMPLE(COLORREF, Color, RGB(0, 0, 0))
	VXP_IMPLEMENT_SIMPLE(COLORREF, BackColor, RGB(0, 0, 0))

private:

	__vxpfont():m_pInterface(NULL){}
	IVXPFont * m_pInterface;
};

/////////////////////////////////////////////////
// class CVXPFont
// This class is to contain interface IVXPFont.
// It has all the same methods and properties
// as the IVXPFont interface to access them seamlessly
// just as though working directly with the COM interface.
/////////////////////////////////////////////////
class CVXPFont:public __vxpfont
{
public:

	// Constructor;
	// Passing false will prevent the constructor from creating
	// a new instance of the interface object.
	CVXPFont(bool bInitialize = true)
	{
		if(bInitialize)
		{
			IVXPFont * pFont;
			::CoCreateInstance(CLSID_VXPFont, NULL, CLSCTX_ALL, IID_IVXPFont, (void**)&pFont);
			Attach(pFont);
		}
	}

	// Desctructor;
	// It will detach and release the interface object
	// automatically when the object goes out of scope.
	~CVXPFont()
	{
		Detach();
	}

	// Attaches an object instance to the class so that all
	// methods and properties of the COM interface would be
	// accessible via properties and methods of the class.
	void Attach(IVXPFont * pFont)
	{
		Detach();
		if(pFont)
			m_pInterface = pFont;
	}

	// Detaches previously attached COM interface,
	// and releases it.
	void Detach()
	{
		if(m_pInterface)
		{
			m_pInterface->Release();
			m_pInterface = NULL;
		}
	}

////////////////
// Properties:
////////////////

	VXP_DECLARE(bool, Transparent)
	VXP_DECLARE(bool, Italic)
	VXP_DECLARE(bool, Underlined)
	VXP_DECLARE(bool, StrikeOut)
	VXP_DECLARE(short, Size)
	VXP_DECLARE(short, Escapement)
	VXP_DECLARE(short, Quality)
	VXP_DECLARE(short, PitchAndFamily)
	VXP_DECLARE(short, Orientation)
	VXP_DECLARE(short, OutPrecision)
	VXP_DECLARE(short, ClipPrecision)
	VXP_DECLARE_READ(HFONT, hFont)
	VXP_DECLARE(VXPFontWeight, Weight)
	VXP_DECLARE(VXPCharSet, CharSet)
	VXP_DECLARE(_bstr_t, Name)
	VXP_DECLARE(COLORREF, Color)
	VXP_DECLARE(COLORREF, BackColor)

////////////
// Methods:
////////////

	void SetDefaults()
	{
		if(m_pInterface)
			m_pInterface->SetDefaults();
	}
	void SetFromHandle(HFONT hFont)
	{
		if(m_pInterface)
			m_pInterface->SetFromHandle((long)hFont);
	}
	void SetFromSystemFont(VXPSysFont FontIndex)
	{
		if(m_pInterface)
			m_pInterface->SetFromSystemFont(FontIndex);
	}

/////////////////////////
// Overloaded operators:
/////////////////////////

	CVXPFont & operator = (CVXPFont & font)
	{
		Transparent = font.Transparent;
		Name = font.Name;
		Size = font.Size;
		Weight = font.Weight;
		Italic = font.Italic;
		Underlined = font.Underlined;
		StrikeOut = font.StrikeOut;
		Orientation = font.Orientation;
		Escapement = font.Escapement;
		CharSet = font.CharSet;
		OutPrecision = font.OutPrecision;
		ClipPrecision = font.ClipPrecision;
		Quality = font.Quality;
		PitchAndFamily = font.PitchAndFamily;
		Color = font.Color;
		BackColor = font.BackColor;
		return *this;
	}
	bool operator == (CVXPFont & font)
	{
		return (Transparent == font.Transparent && Name == font.Name && Size == font.Size && Weight == font.Weight && Italic == font.Italic && Underlined == font.Underlined && StrikeOut == font.StrikeOut && Orientation == font.Orientation && Escapement == font.Escapement && CharSet == font.CharSet && OutPrecision == font.OutPrecision && ClipPrecision == font.ClipPrecision && Quality == font.Quality && PitchAndFamily == font.PitchAndFamily && Color == font.Color && BackColor == font.BackColor);
	}
	bool operator != (CVXPFont & font)
	{
		return (Transparent != font.Transparent || Name != font.Name || Size != font.Size || Weight != font.Weight || Italic != font.Italic || Underlined != font.Underlined || StrikeOut != font.StrikeOut || Orientation != font.Orientation || Escapement != font.Escapement || CharSet != font.CharSet || OutPrecision != font.OutPrecision || ClipPrecision != font.ClipPrecision || Quality != font.Quality || PitchAndFamily != font.PitchAndFamily || Color != font.Color || BackColor != font.BackColor);
	}
};

////////////////////////////////////////
// This is a service class, and should
// not be used by client applications.
////////////////////////////////////////
class __vxpsound
{
friend class CVXPSound;

public:

	VXP_IMPLEMENT_STR(FilePath)
	VXP_IMPLEMENT_STR(Res)
	VXP_IMPLEMENT_STR(Info)
	VXP_IMPLEMENT_SIMPLE(VXPPlayFrom, PlayFrom, pfCustom)
	VXP_IMPLEMENT_LONG(Size)
	VXP_IMPLEMENT_LONG(Volume)
	VXP_IMPLEMENT_LONG(Param)
	VXP_IMPLEMENT_LONG(Balance)
	VXP_IMPLEMENT_LONG(Frequency)
	VXP_IMPLEMENT_LONG(StartPos)
	VXP_IMPLEMENT_LONG(EndPos)
	VXP_IMPLEMENT_LONG(Loop)
	VXP_IMPLEMENT_LONG(Delay)
	VXP_IMPLEMENT_LONG(Pause)
	VXP_IMPLEMENT_BOOL(KeepSettings)
	VXP_IMPLEMENT_HANDLE(HANDLE, Handle)

private:

	__vxpsound():m_pInterface(NULL){}
	IVXPSound * m_pInterface;
};

/////////////////////////////////////////////////
// class CVXPSound
// This class is to contain interface IVXPSound.
// It has all the same methods and properties
// as the IVXPSound interface to access them seamlessly
// just as though working directly with the COM interface.
/////////////////////////////////////////////////
class CVXPSound:public __vxpsound
{
public:

	// Constructor;
	// Passing false will prevent the constructor from creating
	// a new instance of the interface object.
	CVXPSound(bool bInitialize = true)
	{
		if(bInitialize)
		{
			IVXPSound * pSound;
			::CoCreateInstance(CLSID_VXPSound, NULL, CLSCTX_ALL, IID_IVXPSound, (void**)&pSound);
			Attach(pSound);
		}
	}

	// Desctructor;
	// It will detach and release the interface object
	// automatically when the object goes out of scope.
	~CVXPSound()
	{
		Detach();
	}

	// Attaches an object instance to the class so that all
	// methods and properties of the COM interface would be
	// accessible via properties and methods of the class.
	void Attach(IVXPSound * pSound)
	{
		Detach();
		if(pSound)
			m_pInterface = pSound;
	}

	// Detaches previously attached COM interface,
	// and releases it.
	void Detach()
	{
		if(m_pInterface)
		{
			m_pInterface->Release();
			m_pInterface = NULL;
		}
	}

////////////////
// Properties:
////////////////

	VXP_DECLARE(_bstr_t, FilePath)
	VXP_DECLARE(_bstr_t, Res)
	VXP_DECLARE(_bstr_t, Info)
	VXP_DECLARE(VXPPlayFrom, PlayFrom)
	VXP_DECLARE_READ(HANDLE, Handle)
	VXP_DECLARE(long, Size)
	VXP_DECLARE(long, Volume)
	VXP_DECLARE(long, Param)
	VXP_DECLARE(long, Balance)
	VXP_DECLARE(long, Frequency)
	VXP_DECLARE(long, StartPos)
	VXP_DECLARE(long, EndPos)
	VXP_DECLARE(long, Loop)
	VXP_DECLARE(long, Delay)
	VXP_DECLARE(long, Pause)
	VXP_DECLARE(bool, KeepSettings)

////////////
// Methods:
////////////

	void SetDefaults()
	{
		if(m_pInterface)
			m_pInterface->SetDefaults();
	}

/////////////////////////
// Overloaded operators:
/////////////////////////

	CVXPSound & operator = (CVXPSound & sound)
	{
		FilePath = sound.FilePath;
		Res = sound.Res;
		Info = sound.Info;
		PlayFrom = sound.PlayFrom;
		Size = sound.Size;
		Volume = sound.Volume;
		Param = sound.Param;
		Balance = sound.Balance;
		Frequency = sound.Frequency;
		StartPos = sound.StartPos;
		EndPos = sound.EndPos;
		Loop = sound.Loop;
		Delay = sound.Delay;
		Pause = sound.Pause;
		KeepSettings = sound.KeepSettings;
		return *this;
	}
	bool operator == (CVXPSound & sound)
	{
		return (FilePath == sound.FilePath && Res == sound.Res && Info == sound.Info && PlayFrom == sound.PlayFrom && Size == sound.Size && Volume == sound.Volume && Param == sound.Param && Balance == sound.Balance && Frequency == sound.Frequency && StartPos == sound.StartPos && EndPos == sound.EndPos && Loop == sound.Loop && Delay == sound.Delay && Pause == sound.Pause && KeepSettings == sound.KeepSettings);
	}
	bool operator != (CVXPSound & sound)
	{
		return (FilePath != sound.FilePath || Res != sound.Res || Info != sound.Info || PlayFrom != sound.PlayFrom || Size != sound.Size || Volume != sound.Volume || Param != sound.Param || Balance != sound.Balance || Frequency != sound.Frequency || StartPos != sound.StartPos || EndPos != sound.EndPos || Loop != sound.Loop || Delay != sound.Delay || Pause != sound.Pause || KeepSettings != sound.KeepSettings);
	}
};

////////////////////////////////////////
// This is a service class, and should
// not be used by client applications.
////////////////////////////////////////
class __vxpmargins
{
friend class CVXPMargins;

public:

	VXP_IMPLEMENT_SHORT(Left)
	VXP_IMPLEMENT_SHORT(Top)
	VXP_IMPLEMENT_SHORT(Right)
	VXP_IMPLEMENT_SHORT(Bottom)

private:

	__vxpmargins():m_pInterface(NULL){}
	IVXPMargins * m_pInterface;
};

/////////////////////////////////////////////////
// class CVXPMargins
// This class is to contain interface IVXPMargins.
// It has all the same methods and properties
// as the IVXPMargins interface to access them seamlessly
// just as though working directly with the COM interface.
/////////////////////////////////////////////////
class CVXPMargins:public __vxpmargins
{
public:

	// Constructor;
	// Passing false will prevent the constructor from creating
	// a new instance of the interface object.
	CVXPMargins(bool bInitialize = true)
	{
		if(bInitialize)
		{
			IVXPMargins * pMargins;
			::CoCreateInstance(CLSID_VXPMargins, NULL, CLSCTX_ALL, IID_IVXPMargins, (void**)&pMargins);
			Attach(pMargins);
		}
	}

	// Desctructor;
	// It will detach and release the interface object
	// automatically when the object goes out of scope.
	~CVXPMargins()
	{
		Detach();
	}

	// Attaches an object instance to the class so that all
	// methods and properties of the COM interface would be
	// accessible via properties and methods of the class.
	void Attach(IVXPMargins * pMargins)
	{
		Detach();
		if(pMargins)
			m_pInterface = pMargins;
	}

	// Detaches previously attached COM interface,
	// and releases it.
	void Detach()
	{
		if(m_pInterface)
		{
			m_pInterface->Release();
			m_pInterface = NULL;
		}
	}

////////////////
// Properties:
////////////////

	VXP_DECLARE(short, Left)
	VXP_DECLARE(short, Top)
	VXP_DECLARE(short, Right)
	VXP_DECLARE(short, Bottom)

////////////
// Methods:
////////////

	void AdjustMarginsByMin(short MinLeft, short MinTop, short MinRight, short MinBottom)
	{
		if(m_pInterface)
			m_pInterface->AdjustMarginsByMin(MinLeft, MinTop, MinRight, MinBottom);
	}
	void AdjustMarginsByMax(short MaxLeft, short MaxTop, short MaxRight, short MaxBottom)
	{
		if(m_pInterface)
			m_pInterface->AdjustMarginsByMax(MaxLeft, MaxTop, MaxRight, MaxBottom);
	}
	void SetDefaults()
	{
		if(m_pInterface)
			m_pInterface->SetDefaults();
	}

/////////////////////////
// Overloaded operators:
/////////////////////////

	CVXPMargins & operator = (CVXPMargins & margins)
	{
		Left = margins.Left;
		Top = margins.Top;
		Right = margins.Right;
		Bottom = margins.Bottom;
		return *this;
	}
	bool operator == (CVXPMargins & margins)
	{
		return (Left == margins.Left && Top == margins.Top && Right == margins.Right && Bottom == margins.Bottom);
	}
	bool operator != (CVXPMargins & margins)
	{
		return (Left != margins.Left || Top != margins.Top || Right != margins.Right || Bottom != margins.Bottom);
	}
};

////////////////////////////////////////
// This is a service class, and should
// not be used by client applications.
////////////////////////////////////////
class __vxpsettings
{
friend class CVXPSettings;

public:

	VXP_IMPLEMENT_REF(CVXPMargins, margins)
	VXP_IMPLEMENT_REF(CVXPSound, bgsound)
	VXP_IMPLEMENT_REF(CVXPFont, font)
	VXP_IMPLEMENT_SIMPLE(COLORREF, bgColor, RGB(0, 0, 0))

private:

	__vxpsettings():m_pInterface(NULL), m_font(false), m_bgsound(false), m_margins(false){}

	IVXPSettings * m_pInterface;
	CVXPFont m_font;
	CVXPSound m_bgsound;
	CVXPMargins m_margins;
};

/////////////////////////////////////////////////
// class CVXPSettings
// This class is to contain interface IVXPSettings.
// It has all the same methods and properties
// as the IVXPSettings interface to access them seamlessly
// just as though working directly with the COM interface.
// It also contains objects CVXPFont, CVXPSound and CVXPMargins
// accessible through properties font, bgsound and margins
// respectively.
/////////////////////////////////////////////////
class CVXPSettings:public __vxpsettings
{
public:

	// Constructor;
	// Passing false will prevent the constructor from creating
	// a new instance of the interface object.
	CVXPSettings(bool bInitialize = true)
	{
		if(bInitialize)
		{
			IVXPSettings * pSettings;
			::CoCreateInstance(CLSID_VXPSettings, NULL, CLSCTX_ALL, IID_IVXPSettings, (void**)&pSettings);
			Attach(pSettings);
		}
	}

	// Desctructor;
	// It will detach and release the interface object
	// automatically when the object goes out of scope.
	~CVXPSettings()
	{
		Detach();
	}

	// Attaches an object instance to the class so that all
	// methods and properties of the COM interface would be
	// accessible via properties and methods of the class.
	void Attach(IVXPSettings * pSettings)
	{
		Detach();
		if(pSettings)
		{
			m_pInterface = pSettings;

			IVXPFont * pFont;
			pSettings->get_font(&pFont);
			m_font.Attach(pFont);

			IVXPSound * pSound;
			pSettings->get_bgsound(&pSound);
			m_bgsound.Attach(pSound);

			IVXPMargins * pMargins;
			pSettings->get_margins(&pMargins);
			m_margins.Attach(pMargins);
		}
	}

	// Detaches previously attached COM interface,
	// and releases it.
	void Detach()
	{
		if(m_pInterface)
		{
			m_font.Detach();
			m_bgsound.Detach();
			m_margins.Detach();

			m_pInterface->Release();
			m_pInterface = NULL;
		}
	}

////////////////
// Properties:
////////////////

	VXP_DECLARE(COLORREF, bgColor)
	VXP_DECLARE_READ(CVXPFont&, font)
	VXP_DECLARE_READ(CVXPSound&, bgsound)
	VXP_DECLARE_READ(CVXPMargins&, margins)

////////////
// Methods:
////////////

	void SetDefaults()
	{
		if(m_pInterface)
			m_pInterface->SetDefaults();
	}

/////////////////////////
// Overloaded operators:
/////////////////////////

	CVXPSettings & operator = (CVXPSettings & settings)
	{
		bgColor = settings.bgColor;
		m_bgsound = settings.m_bgsound;
		m_font = settings.m_font;
		m_margins = settings.m_margins;
		return *this;
	}
	bool operator == (CVXPSettings & settings)
	{
		return (bgColor == settings.bgColor && m_bgsound == settings.m_bgsound && m_font == settings.m_font && m_margins == settings.m_margins);
	}
	bool operator != (CVXPSettings & settings)
	{
		return (bgColor != settings.bgColor || m_bgsound != settings.m_bgsound || m_font != settings.m_font || m_margins != settings.m_margins);
	}
};

#ifdef VXPLIB_WITH_EVENTS

class CVXPQuickHTML;

////////////////////////////////////////
// This is a service class, and should
// not be used by client applications.
////////////////////////////////////////
interface __vxpquickhtmlevents:public IDispEventImpl<0, __vxpquickhtmlevents, &DIID__IVXPQuickHTMLEvents, &LIBID_VXPLibrary, 1, 0>
{
friend class CVXPQuickHTML;
friend class __vxpquickhtml;

public:

BEGIN_SINK_MAP(__vxpquickhtmlevents)
	SINK_ENTRY_EX(0, DIID__IVXPQuickHTMLEvents, 0x01, OnDocumentGetSize)
	SINK_ENTRY_EX(0, DIID__IVXPQuickHTMLEvents, 0x02, OnDocumentDraw)
	SINK_ENTRY_EX(0, DIID__IVXPQuickHTMLEvents, 0x03, OnSetDefaults)
	SINK_ENTRY_EX(0, DIID__IVXPQuickHTMLEvents, 0x04, OnSourceChanged)
	SINK_ENTRY_EX(0, DIID__IVXPQuickHTMLEvents, 0x05, OnRecalculate)
	SINK_ENTRY_EX(0, DIID__IVXPQuickHTMLEvents, 0x06, OnGetText)
END_SINK_MAP()

private:

	__vxpquickhtmlevents()
	{
		m_pClient = NULL;
	}

	inline HRESULT __stdcall OnSetDefaults();
	inline HRESULT __stdcall OnDocumentGetSize(BSTR DocumentPath, BSTR Param, short * Width, short * Height);
	inline HRESULT __stdcall OnDocumentDraw(BSTR DocumentPath, BSTR Param, long DC, short x, short y, short Width, short Height);
	inline HRESULT __stdcall OnSourceChanged(VARIANT_BOOL IsEmpty);
	inline HRESULT __stdcall OnRecalculate(VARIANT_BOOL Begin, VARIANT_BOOL SourceChanged);
	inline HRESULT __stdcall OnGetText(BSTR FilePath, BSTR Res, long Size, long ID, BSTR * Result);

	CVXPQuickHTML * m_pClient;
};

static _ATL_FUNC_INFO OnStateChangedInfo = {CC_STDCALL, VT_EMPTY, 1, {VT_I4}};

class CVXPTooltip;

////////////////////////////////////////
// This is a service class, and should
// not be used by client applications.
////////////////////////////////////////
interface __vxptooltipevents:public IDispEventImpl<0, __vxptooltipevents, &DIID__IVXPTooltipEvents, &LIBID_VXPLibrary, 1, 0>
{
friend class CVXPTooltip;
friend class __vxptooltip;

public:

BEGIN_SINK_MAP(__vxptooltipevents)
	SINK_ENTRY_INFO(0, DIID__IVXPTooltipEvents, 0x01, OnStateChanged, &OnStateChangedInfo)
	SINK_ENTRY_EX(0, DIID__IVXPTooltipEvents, 0x02, OnSoundPlay)
	SINK_ENTRY_EX(0, DIID__IVXPTooltipEvents, 0x03, OnSoundAdjust)
	SINK_ENTRY_EX(0, DIID__IVXPTooltipEvents, 0x04, OnSoundStop)
END_SINK_MAP()

private:

	__vxptooltipevents()
	{
		m_pClient = NULL;
	}

	inline HRESULT __stdcall OnStateChanged(VXPTooltipState NewState);
	inline HRESULT __stdcall OnSoundPlay(struct IVXPSound * bgsound, VARIANT_BOOL * StartedPlaying );
	inline HRESULT __stdcall OnSoundAdjust(struct IVXPSound * bgsound, float FadingPercent);
	inline HRESULT __stdcall OnSoundStop(struct IVXPSound * bgsound);

	CVXPTooltip * m_pClient;
};

class CVXPTooltipManager;

////////////////////////////////////////
// This is a service class, and should
// not be used by client applications.
////////////////////////////////////////
interface __vxptooltipmanagerevents:public IDispEventImpl<0, __vxptooltipmanagerevents, &DIID__IVXPTooltipManagerEvents, &LIBID_VXPLibrary, 1, 0>
{
friend class CVXPTooltipManager;
friend class __vxptooltipmanager;

public:

BEGIN_SINK_MAP(__vxptooltipmanagerevents)
	SINK_ENTRY_EX(0, DIID__IVXPTooltipManagerEvents, 0x01, OnActivateCustomTooltip)
END_SINK_MAP()

private:

	__vxptooltipmanagerevents()
	{
		m_pClient = NULL;
	}

	inline HRESULT __stdcall OnActivateCustomTooltip(long hWnd, BSTR ClassName, short x, short y, BSTR * Text);

	CVXPTooltipManager * m_pClient;
};

#endif

////////////////////////////////////////
// This is a service class, and should
// not be used by client applications.
////////////////////////////////////////
class __vxpwndtoollist
{
friend class CVXPWndToolList;

public:

	short GetSize()
	{
		if(m_pWndToolList)
		{
			short size;
			m_pWndToolList->get_Size(&size);
			return size;
		}
		return 0;
	}

private:

	__vxpwndtoollist():m_pWndToolList(NULL){}
	IVXPWndToolList * m_pWndToolList;
};

/////////////////////////////////////////////////
// class CVXPWndToolList
// This class is to contain interface IVXPWndToolList.
// It has all the same methods and properties
// as the IVXPWndToolList interface to access them seamlessly
// just as though working directly with the COM interface.
// It is used by class CVXPTooltipManager.
/////////////////////////////////////////////////
class CVXPWndToolList:public __vxpwndtoollist
{
public:

	// Constructor;
	// Passing false will prevent the constructor from creating
	// a new instance of the interface object.
	CVXPWndToolList(bool bInitialize = true)
	{
		if(bInitialize)
		{
			IVXPWndToolList * pWndToolList;
			::CoCreateInstance(CLSID_VXPWndToolList, NULL, CLSCTX_ALL, IID_IVXPWndToolList, (void**)&pWndToolList);
			Attach(pWndToolList);
		}
	}
	
	// Desctructor;
	// It will detach and release the interface object
	// automatically when the object goes out of scope.
	~CVXPWndToolList()
	{
		Detach();
	}

	// Attaches an object instance to the class so that all
	// methods and properties of the COM interface would be
	// accessible via properties and methods of the class.
	void Attach(IVXPWndToolList * pWndToolList)
	{
		Detach();
		if(pWndToolList)
			m_pWndToolList = pWndToolList;
	}

	// Detaches previously attached COM interface,
	// and releases it.
	void Detach()
	{
		if(m_pWndToolList)
		{
			m_pWndToolList->Release();
			m_pWndToolList = NULL;
		}
	}

////////////////
// Properties:
////////////////

	VXP_DECLARE_READ(short, Size)

////////////
// Methods:
////////////

	void Add(HWND hWnd, LPCTSTR ToolTip)
	{
		if(m_pWndToolList && ToolTip)
		{
			_bstr_t tooltip(ToolTip);
			m_pWndToolList->Add((long)hWnd, tooltip);
		}
	}
	bool Modify(HWND hWnd, LPCTSTR ToolTip)
	{
		if(m_pWndToolList && ToolTip)
		{
			_bstr_t tooltip(ToolTip);
			return m_pWndToolList->Modify((long)hWnd, tooltip)?true:false;
		}
		return false;
	}
	bool Remove(HWND hWnd)
	{
		if(m_pWndToolList)
			return m_pWndToolList->Remove((long)hWnd)?true:false;
		return false;
	}
	void Clear()
	{
		if(m_pWndToolList)
			m_pWndToolList->Clear();
	}
	bool HasTool(HWND hWnd)
	{
		if(m_pWndToolList)
			return m_pWndToolList->HasTool((long)hWnd)?true:false;
		return false;
	}
	bool IsActive(HWND hWnd)
	{
		if(m_pWndToolList)
			return m_pWndToolList->IsActive((long)hWnd)?true:false;
		return false;
	}
	bool SetActive(HWND hWnd, bool Active)
	{
		if(m_pWndToolList)
			return m_pWndToolList->SetActive((long)hWnd, Active?TRUE:FALSE)?true:false;
		return false;
	}
};

////////////////////////////////////////
// This is a service class, and should
// not be used by client applications.
////////////////////////////////////////
class __vxpcmdtoollist
{
friend class CVXPCmdToolList;

public:

	short GetSize()
	{
		if(m_pCmdToolList)
		{
			short size;
			m_pCmdToolList->get_Size(&size);
			return size;
		}
		return 0;
	}

private:

	__vxpcmdtoollist():m_pCmdToolList(NULL){}
	IVXPCmdToolList * m_pCmdToolList;
};

/////////////////////////////////////////////////
// class CVXPCmdToolList
// This class is to contain interface IVXPCmdToolList.
// It has all the same methods and properties
// as the IVXPCmdToolList interface to access them seamlessly
// just as though working directly with the COM interface.
// It is used by class CVXPTooltipManager.
/////////////////////////////////////////////////
class CVXPCmdToolList:public __vxpcmdtoollist
{
public:

	// Constructor;
	// Passing false will prevent the constructor from creating
	// a new instance of the interface object.
	CVXPCmdToolList(bool bInitialize = true)
	{
		if(bInitialize)
		{
			IVXPCmdToolList * pCmdToolList;
			::CoCreateInstance(CLSID_VXPCmdToolList, NULL, CLSCTX_ALL, IID_IVXPCmdToolList, (void**)&pCmdToolList);
			Attach(pCmdToolList);
		}
	}

	// Desctructor;
	// It will detach and release the interface object
	// automatically when the object goes out of scope.
	~CVXPCmdToolList()
	{
		Detach();
	}

	// Attaches an object instance to the class so that all
	// methods and properties of the COM interface would be
	// accessible via properties and methods of the class.
	void Attach(IVXPCmdToolList * pCmdToolList)
	{
		Detach();
		if(pCmdToolList)
			m_pCmdToolList = pCmdToolList;
	}

	// Detaches previously attached COM interface,
	// and releases it.
	void Detach()
	{
		if(m_pCmdToolList)
		{
			m_pCmdToolList->Release();
			m_pCmdToolList = NULL;
		}
	}

////////////////
// Properties:
////////////////

	VXP_DECLARE_READ(short, Size)

////////////
// Methods:
////////////

	void Add(long CommandID, LPCTSTR ToolTip)
	{
		if(m_pCmdToolList && ToolTip)
		{
			_bstr_t tooltip(ToolTip);
			m_pCmdToolList->Add(CommandID, tooltip);
		}
	}
	bool Modify(long CommandID, LPCTSTR ToolTip)
	{
		if(m_pCmdToolList && ToolTip)
		{
			_bstr_t tooltip(ToolTip);
			return m_pCmdToolList->Modify(CommandID, tooltip)?true:false;
		}
		return false;
	}
	bool Remove(long CommandID)
	{
		if(m_pCmdToolList)
			return m_pCmdToolList->Remove(CommandID)?true:false;
		return false;
	}
	void Clear()
	{
		if(m_pCmdToolList)
			m_pCmdToolList->Clear();
	}
	bool HasTool(long CommandID)
	{
		if(m_pCmdToolList)
			return m_pCmdToolList->HasTool(CommandID)?true:false;
		return false;
	}
	bool IsActive(long CommandID)
	{
		if(m_pCmdToolList)
			return m_pCmdToolList->IsActive(CommandID)?true:false;
		return false;
	}
	bool SetActive(long CommandID, bool Active)
	{
		if(m_pCmdToolList)
			return m_pCmdToolList->SetActive(CommandID, Active?TRUE:FALSE)?true:false;
		return false;
	}
};

////////////////////////////////////////
// This is a service class, and should
// not be used by client applications.
////////////////////////////////////////
class __vxpquickhtml
{
friend class CVXPQuickHTML;

public:

	VXP_IMPLEMENT_BOOL(Transparent)
	VXP_IMPLEMENT_READ_BOOL(HasBackgroundPicture)
	VXP_IMPLEMENT_READ(short, Width)
	VXP_IMPLEMENT_READ(short, Height)
	VXP_IMPLEMENT_REF(CVXPSettings, settings)
		
private:

	__vxpquickhtml():m_pInterface(NULL), m_settings(false){};
	CVXPSettings m_settings;
	IVXPQuickHTML * m_pInterface;

#ifdef VXPLIB_WITH_EVENTS
	__vxpquickhtmlevents m_sink;
#endif

};

/////////////////////////////////////////////////
// class CVXPQuickHTML - Quick HTML parser;
// This class is to contain interface IVXPQuickHTML.
// It has all the same methods and properties
// as the IVXPQuickHTML interface to access them seamlessly
// just as though working directly with the COM interface.
/////////////////////////////////////////////////
class CVXPQuickHTML:public __vxpquickhtml
{
#ifdef VXPLIB_WITH_EVENTS
friend interface __vxpquickhtmlevents;
#endif
public:

	// Constructor;
	// Passing false will prevent the constructor from creating
	// a new instance of the interface object.
	CVXPQuickHTML(bool bInitialize = true)
	{
		if(bInitialize)
		{
			IVXPQuickHTML * pHTML;
			::CoCreateInstance(CLSID_VXPQuickHTML, NULL, CLSCTX_ALL, IID_IVXPQuickHTML, (void**)&pHTML);
			Attach(pHTML);
		}
	};

	// Desctructor;
	// It will detach and release the interface object
	// automatically when the object goes out of scope.
	~CVXPQuickHTML()
	{
		Detach();
	}

	// Attaches an object instance to the class so that all
	// methods and properties of the COM interface would be
	// accessible via properties and methods of the class.
	void Attach(IVXPQuickHTML * pObject)
	{
		Detach();
		if(pObject)
		{
			m_pInterface = pObject;
#ifdef VXPLIB_WITH_EVENTS
			m_sink.m_pClient = this;
			m_sink.DispEventAdvise(m_pInterface);
#endif
			IVXPSettings * pSettings;
			pObject->get_settings(&pSettings);
			m_settings.Attach(pSettings);
		}
	}

	// Detaches previously attached COM interface,
	// and releases it.
	void Detach()
	{
		if(m_pInterface)
		{
			m_settings.Detach();
#ifdef VXPLIB_WITH_EVENTS
			m_sink.DispEventUnadvise(m_pInterface);
			m_sink.m_pClient = NULL;
#endif
			m_pInterface->Release();
			m_pInterface = NULL;
		}
	}

////////////////
// Properties:
////////////////

	VXP_DECLARE(bool, Transparent)
	VXP_DECLARE_READ(bool, HasBackgroundPicture)
	VXP_DECLARE(short, Width)
	VXP_DECLARE(short, Height)
	VXP_DECLARE(CVXPSettings&, settings)

////////////
// Methods:
////////////

	void RecalculateLayout()
	{
		if(m_pInterface)
			m_pInterface->RecalculateLayout();
	}
	void SetSourceText(LPCTSTR text)
	{
		if(m_pInterface)
		{
			_bstr_t t(text);
			m_pInterface->SetSourceText(t);
		}
	}
	void Draw(HDC dc, short x, short y)
	{
		if(m_pInterface)
			m_pInterface->Draw((long)dc, x, y);
	}
	void DrawStretched(HDC dc, short x, short y, short width, short height)
	{
		if(m_pInterface)
			m_pInterface->DrawStretched((long)dc, x, y, width, height);
	}
	bool DrawBackground(HDC dc, short x, short y, short width, short height, bool DrawBackColor = true)
	{
		if(m_pInterface)
			return m_pInterface->DrawBackground((long)dc, x, y, width, height, DrawBackColor)?true:false;
		return false;
	}
	void AboutBox()
	{
		if(m_pInterface)
			m_pInterface->AboutBox();
	}
	bool SetDefaultRes(LPCTSTR FilePath)
	{
		if(m_pInterface && FilePath)
		{
			_bstr_t fpath(FilePath);
			return m_pInterface->SetDefaultRes(fpath)?true:false;
		}
		return false;
	}
	short AddFilePath(LPCTSTR FilePath)
	{
		if(m_pInterface && FilePath)
		{
			_bstr_t fpath(FilePath);
			return m_pInterface->AddFilePath(fpath);
		}
		return 0;
	}
	_bstr_t GetSpeechText(VXPSpeechFormat format = sfNone, bool bIncludeComments = true, bool bIncludeText = true)
	{
		if(m_pInterface)
			return m_pInterface->GetSpeechText(format, bIncludeComments, bIncludeText);
		return _bstr_t();
	}

protected:

	// Events:
	virtual void OnSetDefaults(){};
	virtual void OnDocumentGetSize(LPCTSTR DocumentPath, LPCTSTR Param, short & Width, short & Height){};
	virtual void OnDocumentDraw(LPCTSTR DocumentPath, LPCTSTR Param, HDC DC, short x, short y, short Width, short Height){};
	virtual void OnSourceChanged(bool IsEmpty){};
	virtual void OnRecalculate(bool Begin, bool SourceChanged){};
	virtual BSTR OnGetText(LPCTSTR FilePath, LPCTSTR Res, long Size, long ID){return NULL;};
};

////////////////////////////////////////
// This is a service class, and should
// not be used by client applications.
////////////////////////////////////////
class __vxptooltip
{
	friend class CVXPTooltip;

public:

	VXP_IMPLEMENT_READ_SIMPLE(VXPTooltipState, State, ttsHidden)
	VXP_IMPLEMENT_SIMPLE(VXPTooltipTheme, Theme, ttDefault)
	VXP_IMPLEMENT_SIMPLE(BYTE, Transparency, 0)
	VXP_IMPLEMENT_SIMPLE(BYTE, Round, 0)
	VXP_IMPLEMENT_HANDLE(HWND, hWnd)
	VXP_IMPLEMENT_SHORT(FadeShow)
	VXP_IMPLEMENT_SHORT(FadeHide)
	VXP_IMPLEMENT_SHORT(FadeSound)
	VXP_IMPLEMENT_BOOL(HasTail)
	VXP_IMPLEMENT_BOOL(HasBorder)
	VXP_IMPLEMENT_BOOL(HasShadow)
	VXP_IMPLEMENT_BOOL(TraceMode)
	VXP_IMPLEMENT_BOOL(ShowWhenEmpty)
	VXP_IMPLEMENT_BOOL(Autohide)

private:

	__vxptooltip():m_pInterface(NULL){}
	IVXPTooltip * m_pInterface;

#ifdef VXPLIB_WITH_EVENTS
	__vxptooltipevents m_sink;
#endif
};

/////////////////////////////////////////////////////////////
// class CVXPTooltip
// This class is to contain interface IVXPTooltip.
// It has all the same methods and properties
// as the IVXPTooltip interface to access them seamlessly
// just as though working directly with the COM interface.
/////////////////////////////////////////////////////////////
class CVXPTooltip:public CVXPQuickHTML, public __vxptooltip
{
#ifdef VXPLIB_WITH_EVENTS
friend interface __vxptooltipevents;
#endif
public:

	// Constructor;
	// Passing false will prevent the constructor from creating
	// a new instance of the interface object.
	CVXPTooltip(bool bInitialize = true):CVXPQuickHTML(false)
	{
		if(bInitialize)
		{
			IVXPTooltip * pTooltip;
			::CoCreateInstance(CLSID_VXPTooltip, NULL, CLSCTX_ALL, IID_IVXPTooltip, (void**)&pTooltip);
			Attach(pTooltip);
		}
	};

	// Desctructor;
	// It will detach and release the interface object
	// automatically when the object goes out of scope.
	~CVXPTooltip()
	{
		Detach();
	}

	// Attaches an object instance to the class so that all
	// methods and properties of the COM interface would be
	// accessible via properties and methods of the class.
	void Attach(IVXPTooltip * pObject)
	{
		Detach();
		if(pObject)
		{
			__vxptooltip::m_pInterface = pObject;

#ifdef VXPLIB_WITH_EVENTS
			__vxptooltip::m_sink.m_pClient = this;
			__vxptooltip::m_sink.DispEventAdvise(__vxptooltip::m_pInterface);
#endif
			IVXPQuickHTML * pHTML;
			__vxptooltip::m_pInterface->get_html(&pHTML);
			CVXPQuickHTML::Attach(pHTML);
		}
	}

	// Detaches previously attached COM interface,
	// and releases it.
	void Detach()
	{
		if(__vxptooltip::m_pInterface)
		{
#ifdef VXPLIB_WITH_EVENTS
			__vxptooltip::m_sink.DispEventUnadvise(__vxptooltip::m_pInterface);
			__vxptooltip::m_sink.m_pClient = NULL;
#endif
			__vxptooltip::m_pInterface->Release();
			__vxptooltip::m_pInterface = NULL;
		}
	}

////////////////
// Properties:
////////////////

	VXP_DECLARE_READ(VXPTooltipState, State)
	VXP_DECLARE(VXPTooltipTheme, Theme)
	VXP_DECLARE(bool, Autohide)
	VXP_DECLARE(short, FadeShow)
	VXP_DECLARE(short, FadeHide)
	VXP_DECLARE(short, FadeSound)
	VXP_DECLARE(BYTE, Transparency)
	VXP_DECLARE(BYTE, Round)
	VXP_DECLARE(bool, HasTail)
	VXP_DECLARE(bool, HasBorder)
	VXP_DECLARE(bool, HasShadow)
	VXP_DECLARE(bool, TraceMode)
	VXP_DECLARE(bool, ShowWhenEmpty)
	VXP_DECLARE_READ(HWND, hWnd)
	VXP_DECLARE(COLORREF, BorderColor)

////////////
// Methods:
////////////

	void Show()
	{
		if(__vxptooltip::m_pInterface)
			__vxptooltip::m_pInterface->Show();
	}
	void ShowAt(short x, short y)
	{
		if(__vxptooltip::m_pInterface)
			__vxptooltip::m_pInterface->ShowAt(x, y);
	}
	void Hide(bool NoFadingEffect = false)
	{
		if(__vxptooltip::m_pInterface)
			__vxptooltip::m_pInterface->Hide(NoFadingEffect);
	}

protected:

	// Events:
	virtual void OnStateChanged(VXPTooltipState NewState){};
	virtual bool OnSoundPlay(CVXPSound & bgsound){return false;};
	virtual void OnSoundAdjust(CVXPSound & bgsound, float FadingPercent){};
	virtual void OnSoundStop(CVXPSound & bgsound){};
};

////////////////////////////////////////
// This is a service class, and should
// not be used by client applications.
////////////////////////////////////////
class __vxptooltipmanager
{
friend class CVXPTooltipManager;
public:

	VXP_IMPLEMENT_REF(CVXPCmdToolList, commands)
	VXP_IMPLEMENT_REF(CVXPWndToolList, windows)
	VXP_IMPLEMENT_SHORT(ShowDelay)
	VXP_IMPLEMENT_SHORT(HideDelay)
	VXP_IMPLEMENT_BOOL(Active)
	VXP_IMPLEMENT_BOOL(DebugMode)
	VXP_IMPLEMENT_BOOL(sfdMenuItems)
	VXP_IMPLEMENT_BOOL(sfdToolbarButtons)

private:

	__vxptooltipmanager():m_pInterface(NULL), m_windows(false), m_commands(false){}

	IVXPTooltipManager * m_pInterface;
	CVXPWndToolList m_windows;
	CVXPCmdToolList m_commands;

#ifdef VXPLIB_WITH_EVENTS
	__vxptooltipmanagerevents m_sink;
#endif

};

/////////////////////////////////////////////////////////////
// class CVXPTooltipManager
// This class is to contain interface IVXPTooltipManager.
// It has all the same methods and properties
// as the IVXPTooltipManager interface to access them seamlessly
// just as though working directly with the COM interface.
/////////////////////////////////////////////////////////////
class CVXPTooltipManager:public CVXPTooltip, public __vxptooltipmanager
{
#ifdef VXPLIB_WITH_EVENTS
friend interface __vxptooltipmanagerevents;
#endif

public:

	// Constructor;
	// Passing false will prevent the constructor from creating
	// a new instance of the interface object.
	CVXPTooltipManager(bool bInitialize = true):CVXPTooltip(false)
	{
		if(bInitialize)
		{
			IVXPTooltipManager * pManager;
			::CoCreateInstance(CLSID_VXPTooltipManager, NULL, CLSCTX_ALL, IID_IVXPTooltipManager, (void**)&pManager);
			Attach(pManager);
		}
	};

	// Desctructor;
	// It will detach and release the interface object
	// automatically when the object goes out of scope.
	~CVXPTooltipManager()
	{
		Detach();
	}

	// Attaches an object instance to the class so that all
	// methods and properties of the COM interface would be
	// accessible via properties and methods of the class.
	void Attach(IVXPTooltipManager * pObject)
	{
		Detach();
		if(pObject)
		{
			__vxptooltipmanager::m_pInterface = pObject;
#ifdef VXPLIB_WITH_EVENTS
			__vxptooltipmanager::m_sink.m_pClient = this;
			__vxptooltipmanager::m_sink.DispEventAdvise(__vxptooltipmanager::m_pInterface);
#endif
			IVXPTooltip * pTooltip;
			pObject->get_tool(&pTooltip);
			CVXPTooltip::Attach(pTooltip);

			IVXPWndToolList * pWndToolList;
			pObject->get_windows(&pWndToolList);
			m_windows.Attach(pWndToolList);

			IVXPCmdToolList * pCmdToolList;
			pObject->get_commands(&pCmdToolList);
			m_commands.Attach(pCmdToolList);
		}
	}

	// Detaches previously attached COM interface,
	// and releases it.
	void Detach()
	{
		if(__vxptooltipmanager::m_pInterface)
		{
#ifdef VXPLIB_WITH_EVENTS
			__vxptooltipmanager::m_sink.DispEventUnadvise(__vxptooltipmanager::m_pInterface);
			__vxptooltipmanager::m_sink.m_pClient = NULL;
#endif
			__vxptooltipmanager::m_pInterface->Release();
			__vxptooltipmanager::m_pInterface = NULL;
		}
	}

////////////////
// Properties:
////////////////

	VXP_DECLARE_READ(CVXPWndToolList&, windows)
	VXP_DECLARE_READ(CVXPCmdToolList&, commands)
	VXP_DECLARE(short, ShowDelay)
	VXP_DECLARE(short, HideDelay)
	VXP_DECLARE(bool, Active)
	VXP_DECLARE(bool, DebugMode)
	VXP_DECLARE(bool, sfdMenuItems)
	VXP_DECLARE(bool, sfdToolbarButtons)

////////////
// Methods:
////////////

	void UpdateContextMenuOwner(HWND hWnd)
	{
		if(__vxptooltipmanager::m_pInterface)
			__vxptooltipmanager::m_pInterface->UpdateContextMenuOwner((long)hWnd);
	}
	void AddMenuClass(LPCTSTR ClassName)
	{
		if(__vxptooltipmanager::m_pInterface && ClassName)
		{
			_bstr_t cname(ClassName);
			__vxptooltipmanager::m_pInterface->AddMenuClass(cname);
		}
	}
	void AddToolbarClass(LPCTSTR ClassName)
	{
		if(__vxptooltipmanager::m_pInterface && ClassName)
		{
			_bstr_t cname(ClassName);
			__vxptooltipmanager::m_pInterface->AddToolbarClass(cname);
		}
	}
	void MarkDirty()
	{
		if(__vxptooltipmanager::m_pInterface)
			__vxptooltipmanager::m_pInterface->MarkDirty();
	}

protected:

	// Events:
	virtual BSTR OnActivateCustomTooltip(HWND hWnd, LPCTSTR ClassName, short x, short y){return NULL;};
};

#ifdef VXPLIB_WITH_EVENTS

HRESULT __stdcall __vxpquickhtmlevents::OnSetDefaults()
{
	if(m_pClient)
		m_pClient->OnSetDefaults();
	return S_OK;
};

HRESULT __stdcall __vxpquickhtmlevents::OnDocumentGetSize(BSTR DocumentPath, BSTR Param, short * Width, short * Height)
{
	if(m_pClient)
	{
		_bstr_t doc_path(DocumentPath);
		_bstr_t param(Param);
		m_pClient->OnDocumentGetSize(doc_path, param, *Width, *Height);
	}
	return S_OK;
}

HRESULT __stdcall __vxpquickhtmlevents::OnDocumentDraw(BSTR DocumentPath, BSTR Param, long DC, short x, short y, short Width, short Height)
{
	if(m_pClient)
	{
		_bstr_t doc_path(DocumentPath);
		_bstr_t param(Param);
		m_pClient->OnDocumentDraw(doc_path, param, (HDC)DC, x, y, Width, Height);
	}
	return S_OK;
}

HRESULT __stdcall __vxpquickhtmlevents::OnSourceChanged(VARIANT_BOOL IsEmpty)
{
	if(m_pClient)
		m_pClient->OnSourceChanged(IsEmpty?true:false);
	return S_OK;
}

HRESULT __stdcall __vxpquickhtmlevents::OnRecalculate(VARIANT_BOOL Begin, VARIANT_BOOL SourceChanged)
{
	if(m_pClient)
		m_pClient->OnRecalculate(Begin?true:false, SourceChanged?true:false);
	return S_OK;
}

HRESULT __stdcall __vxpquickhtmlevents::OnGetText(BSTR FilePath, BSTR Res, long Size, long ID, BSTR * Result)
{
	if(m_pClient)
	{
		_bstr_t file_path(FilePath);
		_bstr_t res(Res);
		*Result = m_pClient->OnGetText(file_path, res, Size, ID);
	}
	return S_OK;
}

HRESULT __stdcall __vxptooltipevents::OnStateChanged(VXPTooltipState NewState)
{
	if(m_pClient)
		m_pClient->OnStateChanged(NewState);
	return S_OK;
}

HRESULT __stdcall __vxptooltipevents::OnSoundPlay(IVXPSound * bgsound, VARIANT_BOOL * StartedPlaying)
{
	if(m_pClient)
		*StartedPlaying = m_pClient->OnSoundPlay(m_pClient->settings.bgsound)?TRUE:FALSE;
	return S_OK;
}

HRESULT __stdcall __vxptooltipevents::OnSoundAdjust(IVXPSound * bgsound, float FadingPercent)
{
	if(m_pClient)
		m_pClient->OnSoundAdjust(m_pClient->settings.bgsound, FadingPercent);
	return S_OK;
}

HRESULT __stdcall __vxptooltipevents::OnSoundStop(IVXPSound * bgsound)
{
	if(m_pClient)
		m_pClient->OnSoundStop(m_pClient->settings.bgsound);
	return S_OK;
}

HRESULT __stdcall __vxptooltipmanagerevents::OnActivateCustomTooltip(long hWnd, BSTR ClassName, short x, short y, BSTR * Text)
{
	if(m_pClient)
	{
		_bstr_t class_name(ClassName);
		*Text = m_pClient->OnActivateCustomTooltip((HWND)hWnd, class_name, x, y);
	}
	return S_OK;
}

#endif

#pragma warning(pop)

#endif
